@@ -92,10 +92,13 @@ Nepomuk::ServerConfigModule::ServerConfigModule( QWidget* parent, const QVariant
     setAboutData(about);
     setButtons(Help|Apply|Default);
     setupUi( this );
+    m_editStrigiExcludeFilters->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
 
     m_folderModel = new FolderSelectionModel( m_viewIndexFolders );
     m_viewIndexFolders->setModel( m_folderModel );
     m_viewIndexFolders->setHeaderHidden( true );
+    m_viewIndexFolders->header()->setStretchLastSection( false );
+    m_viewIndexFolders->header()->setResizeMode( QHeaderView::ResizeToContents );
     m_viewIndexFolders->setRootIsDecorated( true );
     m_viewIndexFolders->setAnimated( true );
     m_viewIndexFolders->setRootIndex( m_folderModel->setRootPath( QDir::rootPath() ) );
@@ -110,6 +113,8 @@ Nepomuk::ServerConfigModule::ServerConfigModule( QWidget* parent, const QVariant
              this, SLOT( changed() ) );
     connect( m_editStrigiExcludeFilters, SIGNAL( changed() ),
              this, SLOT( changed() ) );
+    connect( m_sliderMemoryUsage, SIGNAL( valueChanged(int) ),
+             this, SLOT( changed() ) );
     connect( m_checkShowHiddenFolders, SIGNAL( toggled( bool ) ),
              m_folderModel, SLOT( setHiddenFoldersShown( bool ) ) );
 
@@ -159,6 +164,12 @@ void Nepomuk::ServerConfigModule::load()
     m_folderModel->setFolders( strigiConfig.group( "General" ).readPathEntry( "folders", defaultFolders() ),
                                strigiConfig.group( "General" ).readPathEntry( "exclude folders", QStringList() ) );
     m_editStrigiExcludeFilters->setItems( strigiConfig.group( "General" ).readEntry( "exclude filters", defaultExcludeFilters() ) );
+    m_checkIndexRemovableMedia->setChecked( strigiConfig.group( "General" ).readEntry( "index newly mounted", false ) );
+
+    KConfig serverConfig( "nepomukserverrc" );
+    const int maxMem = qMax( 20, serverConfig.group( "main Settings" ).readEntry( "Maximum memory", 50 ) );
+    m_sliderMemoryUsage->setValue( maxMem );
+    m_editMemoryUsage->setValue( maxMem );
 
     // make sure we do not have a hidden folder to expand which would make QFileSystemModel crash
     // + it would be weird to have a hidden folder indexed but not shown
@@ -175,7 +186,7 @@ void Nepomuk::ServerConfigModule::load()
     foreach( const QString& dir, m_folderModel->includeFolders() + m_folderModel->excludeFolders() ) {
         expandRecursively( m_folderModel->index( dir ), m_viewIndexFolders );
     }
-
+    groupBox->setEnabled(m_checkEnableNepomuk->isChecked());
     recreateStrigiInterface();
     slotUpdateStrigiStatus();
     emit changed(false);
@@ -197,6 +208,7 @@ void Nepomuk::ServerConfigModule::save()
     KConfig config( "nepomukserverrc" );
     config.group( "Basic Settings" ).writeEntry( "Start Nepomuk", m_checkEnableNepomuk->isChecked() );
     config.group( "Service-nepomukstrigiservice" ).writeEntry( "autostart", m_checkEnableStrigi->isChecked() );
+    config.group( "main Settings" ).writeEntry( "Maximum memory", m_sliderMemoryUsage->value() );
 
 
     // 2. update Strigi config
@@ -205,6 +217,7 @@ void Nepomuk::ServerConfigModule::save()
     strigiConfig.group( "General" ).writePathEntry( "exclude folders", excludeFolders );
     strigiConfig.group( "General" ).writeEntry( "exclude filters", m_editStrigiExcludeFilters->items() );
     strigiConfig.group( "General" ).writeEntry( "index hidden folders", m_checkShowHiddenFolders->isChecked() );
+    strigiConfig.group( "General" ).writeEntry( "index newly mounted", m_checkIndexRemovableMedia->isChecked() );
 
 
     // 3. update the current state of the nepomuk server
