@@ -3,7 +3,7 @@
  * $Id: sourceheader 511311 2006-02-19 14:51:05Z trueg $
  *
  * This file is part of the Nepomuk KDE project.
- * Copyright (C) 2006-2008 Sebastian Trueg <trueg@kde.org>
+ * Copyright (C) 2006-2010 Sebastian Trueg <trueg@kde.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -13,24 +13,18 @@
  */
 
 #include "repository.h"
-#include "nepomukstorage-config.h"
 #include "modelcopyjob.h"
 
-#ifdef HAVE_CLUCENE
-#include "cluceneanalyzer.h"
-#endif
-
 #include <Soprano/Backend>
+#include <Soprano/PluginManager>
 #include <Soprano/Global>
 #include <Soprano/Version>
 #include <Soprano/StorageModel>
 #include <Soprano/Error/Error>
 #include <Soprano/Vocabulary/RDF>
 
-#ifdef HAVE_SOPRANO_INDEX
-#include <Soprano/Index/IndexFilterModel>
-#include <Soprano/Index/CLuceneIndex>
-#endif
+#define USING_SOPRANO_NRLMODEL_UNSTABLE_API
+#include <Soprano/NRLModel>
 
 #include <KStandardDirs>
 #include <KDebug>
@@ -39,8 +33,10 @@
 #include <KLocale>
 #include <KNotification>
 #include <KIcon>
+#include <KIO/DeleteJob>
 
 #include <QtCore/QTimer>
+#include <QtCore/QFile>
 #include <QtCore/QThread>
 #include <QtCore/QCoreApplication>
 
@@ -50,28 +46,6 @@ namespace {
     {
         return KStandardDirs::locateLocal( "data", "nepomuk/repository/" + repositoryId + '/' );
     }
-
-#if defined(HAVE_SOPRANO_INDEX) && defined(HAVE_CLUCENE) && SOPRANO_IS_VERSION(2,1,64)
-    class RebuildIndexThread : public QThread
-    {
-    public:
-        RebuildIndexThread( Soprano::Index::IndexFilterModel* model )
-            : m_model( model ) {
-        }
-
-        void run() {
-            m_model->rebuildIndex();
-        }
-
-    private:
-        Soprano::Index::IndexFilterModel* m_model;
-    };
-
-    // The index version that is written by this implementation. Increase this if a major bug
-    // was fixed in the CLucene index or new information needs to be written into the index
-    // (for example using Soprano::IndexFilterModel::addForceIndexPredicate.)
-    const int s_indexVersion = 2;
-#endif
 }
 
 
@@ -79,37 +53,29 @@ Nepomuk::Repository::Repository( const QString& name )
     : m_name( name ),
       m_state( CLOSED ),
       m_model( 0 ),
-      m_analyzer( 0 ),
-      m_index( 0 ),
-      m_indexModel( 0 )
+      m_modelCopyJob( 0 )
 {
 }
 
 
 Nepomuk::Repository::~Repository()
 {
+    kDebug() << m_name;
     close();
 }
 
 
 void Nepomuk::Repository::close()
 {
-    if ( m_state == OPEN ) {
-#ifdef HAVE_SOPRANO_INDEX
-        delete m_indexModel;
-        delete m_index;
-        m_indexModel = 0;
-        m_index = 0;
-#ifdef HAVE_CLUCENE
-        delete m_analyzer;
-        m_analyzer = 0;
-#endif
-#endif
-        delete m_model;
-        m_model = 0;
+    kDebug() << m_name;
 
-        m_state = CLOSED;
-    }
+    delete m_modelCopyJob;
+    m_modelCopyJob = 0;
+
+    delete m_model;
+    m_model = 0;
+
+    m_state = CLOSED;
 }
 
 
@@ -119,10 +85,29 @@ void Nepomuk::Repository::open()
 
     m_state = OPENING;
 
-    // get used backend
+    // get backend
     // =================================
-    const Soprano::Backend* backend = activeSopranoBackend();
-    if ( !backend ) {
+    m_backend = Soprano::PluginManager::instance()->discoverBackendByName( QLatin1String( "virtuosobackend" ) );
+    if ( !m_backend ) {
+        KNotification::event( "failedToStart",
+                              i18nc("@info - notification message",
+                                    "Nepomuk Semantic Desktop needs the Virtuoso RDF server to store its data. "
+                                    "Installing the Virtuoso Soprano plugin is mandatory for using Nepomuk." ),
+                              KIcon( "application-exit" ).pixmap( 32, 32 ),
+                              0,
+                              KNotification::Persistent );
+        m_state = CLOSED;
+        emit opened( this, false );
+        return;
+    }
+    else if ( !m_backend->isAvailable() ) {
+        KNotification::event( "failedToStart",
+                              i18nc("@info - notification message",
+                                    "Nepomuk Semantic Desktop needs the Virtuoso RDF server to store its data. "
+                                    "Installing the Virtuoso server and ODBC driver is mandatory for using Nepomuk." ),
+                              KIcon( "application-exit" ).pixmap( 32, 32 ),
+                              0,
+                              KNotification::Persistent );
         m_state = CLOSED;
         emit opened( this, false );
         return;
@@ -131,32 +116,42 @@ void Nepomuk::Repository::open()
     // read config
     // =================================
     KConfigGroup repoConfig = KSharedConfig::openConfig( "nepomukserverrc" )->group( name() + " Settings" );
-    QString oldBackendName = repoConfig.readEntry( "Used Soprano Backend", backend->pluginName() );
+    QString oldBackendName = repoConfig.readEntry( "Used Soprano Backend", m_backend->pluginName() );
     QString oldBasePath = repoConfig.readPathEntry( "Storage Dir", QString() ); // backward comp: empty string means old storage path
+    Soprano::BackendSettings settings = readVirtuosoSettings();
 
     // If possible we want to keep the old storage path. exception: oldStoragePath is empty. In that case we stay backwards
-    // compatible and convert the data to the new default location createStoragePath( name ) + "data/" + backend->pluginName()
+    // compatible and convert the data to the new default location createStoragePath( name ) + "data/" + m_backend->pluginName()
     //
     // If we have a proper oldStoragePath and a different backend we use the oldStoragePath as basePath
-    // newDataPath = oldStoragePath + "data/" + backend->pluginName()
+    // newDataPath = oldStoragePath + "data/" + m_backend->pluginName()
     // oldDataPath = oldStoragePath + "data/" + oldBackendName
 
 
     // create storage paths
     // =================================
     m_basePath = oldBasePath.isEmpty() ? createStoragePath( name() ) : oldBasePath;
-    QString indexPath = m_basePath + "index";
-    QString storagePath = m_basePath + "data/" + backend->pluginName();
+    QString storagePath = m_basePath + "data/" + m_backend->pluginName();
 
-    KStandardDirs::makeDir( indexPath );
-    KStandardDirs::makeDir( storagePath );
+    if ( !KStandardDirs::makeDir( storagePath ) ) {
+        kDebug() << "Failed to create storage folder" << storagePath;
+        m_state = CLOSED;
+        emit opened( this, false );
+        return;
+    }
 
     kDebug() << "opening repository '" << name() << "' at '" << m_basePath << "'";
 
+    // remove old pre 4.4 clucene index
+    // =================================
+    if ( QFile::exists( m_basePath + QLatin1String( "index" ) ) ) {
+        KIO::del( m_basePath + QLatin1String( "index" ) );
+    }
 
     // open storage
     // =================================
-    m_model = backend->createModel( QList<Soprano::BackendSetting>() << Soprano::BackendSetting( Soprano::BackendOptionStorageDir, storagePath ) );
+    Soprano::settingInSettings( settings, Soprano::BackendOptionStorageDir ).setValue( storagePath );
+    m_model = m_backend->createModel( settings );
     if ( !m_model ) {
         kDebug() << "Unable to create model for repository" << name();
         m_state = CLOSED;
@@ -166,37 +161,13 @@ void Nepomuk::Repository::open()
 
     kDebug() << "Successfully created new model for repository" << name();
 
-#if defined(HAVE_SOPRANO_INDEX) && defined(HAVE_CLUCENE)
-    m_analyzer = new CLuceneAnalyzer();
-    m_index = new Soprano::Index::CLuceneIndex( m_analyzer );
+    // create a NRLModel for those extra nice features
+    // =================================
+    Soprano::NRLModel* nrlModel = new Soprano::NRLModel( m_model );
+    nrlModel->setParent(this); // memory management
+    nrlModel->setEnableQueryPrefixExpansion( true );
 
-    if ( m_index->open( indexPath, true ) ) {
-        kDebug() << "Successfully created new index for repository" << name();
-        m_indexModel = new Soprano::Index::IndexFilterModel( m_index, m_model );
-
-        // FIXME: find a good value here
-        m_indexModel->setTransactionCacheSize( 100 );
-
-#if SOPRANO_IS_VERSION(2,1,64)
-        m_indexModel->addForceIndexPredicate( Soprano::Vocabulary::RDF::type() );
-#endif
-
-        setParentModel( m_indexModel );
-    }
-    else {
-        kDebug() << "Unable to open CLucene index for repo '" << name() << "': " << m_index->lastError();
-        delete m_index;
-        delete m_model;
-        m_index = 0;
-        m_model = 0;
-
-        m_state = CLOSED;
-        emit opened( this, false );
-        return;
-    }
-#else
-    setParentModel( m_model );
-#endif
+    setParentModel( nrlModel );
 
     // check if we have to convert
     // =================================
@@ -205,10 +176,10 @@ void Nepomuk::Repository::open()
     // if the backend changed we convert
     // in case only the storage dir changes we normally would not have to convert but
     // it is just simpler this way
-    if ( oldBackendName != backend->pluginName() ||
+    if ( oldBackendName != m_backend->pluginName() ||
          oldBasePath.isEmpty() ) {
 
-        kDebug() << "Previous backend:" << oldBackendName << "- new backend:" << backend->pluginName();
+        kDebug() << "Previous backend:" << oldBackendName << "- new backend:" << m_backend->pluginName();
         kDebug() << "Old path:" << oldBasePath << "- new path:" << m_basePath;
 
         if ( oldBasePath.isEmpty() ) {
@@ -234,18 +205,29 @@ void Nepomuk::Repository::open()
 
                 convertingData = true;
                 // No need to use the index filter as it already contains the data
-                ModelCopyJob* copyJob = new ModelCopyJob( oldModel, m_model, this );
-                connect( copyJob, SIGNAL( result( KJob* ) ), this, SLOT( copyFinished( KJob* ) ) );
-                copyJob->start();
+                m_modelCopyJob = new ModelCopyJob( oldModel, m_model, this );
+                connect( m_modelCopyJob, SIGNAL( result( KJob* ) ), this, SLOT( copyFinished( KJob* ) ) );
+                m_modelCopyJob->start();
             }
             else {
                 m_state = OPEN;
             }
         }
         else {
-            // FIXME: inform the user
-            kDebug( 300002 ) << "Unable to convert old model.";
-            m_state = OPEN;
+            kDebug( 300002 ) << "Unable to convert old model: cound not load old backend" << oldBackendName;
+            KNotification::event( "convertingNepomukDataFailed",
+                                  i18nc("@info - notification message",
+                                        "Nepomuk was not able to find the configured database backend '%1'. "
+                                        "Existing data can thus not be accessed. "
+                                        "For data security reasons Nepomuk will be disabled until "
+                                        "the situation has been resolved manually.",
+                                        oldBackendName ),
+                                  KIcon( "nepomuk" ).pixmap( 32, 32 ),
+                                  0,
+                                  KNotification::Persistent );
+            m_state = CLOSED;
+            emit opened( this, false );
+            return;
         }
     }
     else {
@@ -258,14 +240,12 @@ void Nepomuk::Repository::open()
     // do not save when converting yet. If converting is cancelled we would loose data.
     // this way conversion is restarted the next time
     if ( !convertingData ) {
-        repoConfig.writeEntry( "Used Soprano Backend", backend->pluginName() );
+        repoConfig.writeEntry( "Used Soprano Backend", m_backend->pluginName() );
         repoConfig.writePathEntry( "Storage Dir", m_basePath );
         repoConfig.sync(); // even if we crash the model has been created
 
         if( m_state == OPEN ) {
-            if ( !rebuildIndexIfNecessary() ) {
-                emit opened( this, true );
-            }
+            emit opened( this, true );
         }
     }
     else {
@@ -277,34 +257,23 @@ void Nepomuk::Repository::open()
 }
 
 
-void Nepomuk::Repository::rebuildingIndexFinished()
-{
-#if defined(HAVE_SOPRANO_INDEX) && defined(HAVE_CLUCENE) && SOPRANO_IS_VERSION(2,1,64)
-    KNotification::event( "rebuldingNepomukIndexDone",
-                          i18nc("@info - notification message",
-                                "Rebuilding Nepomuk full text search index for new features done."),
-                          KIcon( "nepomuk" ).pixmap( 32, 32 ) );
-
-    // save our new settings
-    KConfigGroup repoConfig = KSharedConfig::openConfig( "nepomukserverrc" )->group( name() + " Settings" );
-    repoConfig.writeEntry( "index version", s_indexVersion );
-
-    // inform that we are open and done
-    m_state = OPEN;
-    emit opened( this, true );
-#endif
-}
-
-
 void Nepomuk::Repository::copyFinished( KJob* job )
 {
+    m_modelCopyJob = 0;
+
     if ( job->error() ) {
         KNotification::event( "convertingNepomukDataFailed",
                               i18nc("@info - notification message",
-                                    "Converting Nepomuk data to the new backend failed. Data may still be recovered manually though."),
-                                    KIcon( "nepomuk" ).pixmap( 32, 32 ) );
+                                    "Converting Nepomuk data to the new backend failed. "
+                                    "For data security reasons Nepomuk will be disabled until "
+                                    "the situation has been resolved manually."),
+                              KIcon( "nepomuk" ).pixmap( 32, 32 ),
+                              0,
+                              KNotification::Persistent );
 
         kDebug( 300002 ) << "Converting old model failed.";
+        m_state = CLOSED;
+        emit opened( this, false );
     }
     else {
         KNotification::event( "convertingNepomukDataDone",
@@ -320,73 +289,46 @@ void Nepomuk::Repository::copyFinished( KJob* job )
 
         // cleanup the actual data
         m_oldStorageBackend->deleteModelData( QList<Soprano::BackendSetting>() << Soprano::BackendSetting( Soprano::BackendOptionStorageDir, m_oldStoragePath ) );
+        m_oldStorageBackend = 0;
 
         // save our new settings
         KConfigGroup repoConfig = KSharedConfig::openConfig( "nepomukserverrc" )->group( name() + " Settings" );
-        repoConfig.writeEntry( "Used Soprano Backend", activeSopranoBackend()->pluginName() );
+        repoConfig.writeEntry( "Used Soprano Backend", m_backend->pluginName() );
         repoConfig.writePathEntry( "Storage Dir", m_basePath );
         repoConfig.sync();
 
-        if ( rebuildIndexIfNecessary() ) {
-            // opened will be emitted in rebuildingIndexFinished
-            return;
-        }
+        m_state = OPEN;
+        emit opened( this, true );
     }
-
-    // although converting might have failed, the new model is open anyway
-    m_state = OPEN;
-    emit opened( this, true );
 }
 
 
-void Nepomuk::Repository::optimize()
+QString Nepomuk::Repository::usedSopranoBackend() const
 {
-    QTimer::singleShot( 0, this, SLOT( slotDoOptimize() ) );
+    return m_backend->pluginName();
 }
 
 
-void Nepomuk::Repository::slotDoOptimize()
+Soprano::BackendSettings Nepomuk::Repository::readVirtuosoSettings() const
 {
-#ifdef HAVE_SOPRANO_INDEX
-#if SOPRANO_IS_VERSION(2,1,60)
-    m_index->optimize();
-#endif
-#endif
-}
+    Soprano::BackendSettings settings;
 
-
-bool Nepomuk::Repository::rebuildIndexIfNecessary()
-{
-#if defined(HAVE_SOPRANO_INDEX) && defined(HAVE_CLUCENE) && SOPRANO_IS_VERSION(2,1,64)
     KConfigGroup repoConfig = KSharedConfig::openConfig( "nepomukserverrc" )->group( name() + " Settings" );
-    if( repoConfig.readEntry( "index version", 1 ) < s_indexVersion ) {
-        KNotification::event( "rebuldingNepomukIndex",
-                              i18nc("@info - notification message",
-                                    "Rebuilding Nepomuk full text search index for new features. This will only be done once and might take a while."),
-                                    KIcon( "nepomuk" ).pixmap( 32, 32 ) );
-        RebuildIndexThread* rit = new RebuildIndexThread( m_indexModel );
-        connect( rit, SIGNAL( finished() ), this, SLOT( rebuildingIndexFinished() ) );
-        connect( rit, SIGNAL( finished() ), rit, SLOT( deleteLater() ) );
-        rit->start();
-        return true;
-    }
-#endif
-    return false;
-}
+    const int maxMem = repoConfig.readEntry( "Maximum memory", 50 );
 
+    // below NumberOfBuffers=400 virtuoso crashes (at least on amd64)
+    settings << Soprano::BackendSetting( "buffers", qMax( 4, maxMem-30 )*100 );
 
-const Soprano::Backend* Nepomuk::Repository::activeSopranoBackend()
-{
-    QString backendName = KSharedConfig::openConfig( "nepomukserverrc" )->group( "Basic Settings" ).readEntry( "Soprano Backend", "sesame2" );
-    const Soprano::Backend* backend = ::Soprano::discoverBackendByName( backendName );
-    if ( !backend ) {
-        kDebug() << "(Nepomuk::Core::Core) could not find backend" << backendName << ". Falling back to default.";
-        backend = ::Soprano::usedBackend();
-    }
-    if ( !backend ) {
-        kDebug() << "(Nepomuk::Core::Core) could not find a backend.";
-    }
-    return backend;
+    // make a checkpoint every 10 minutes to minimize the startup time
+    settings << Soprano::BackendSetting( "CheckpointInterval", 10 );
+
+    // lower the minimum transaction log size to make sure the checkpoints are actually executed
+    settings << Soprano::BackendSetting( "MinAutoCheckpointSize", 200000 );
+
+    // alwyays index literals
+    settings << Soprano::BackendSetting( "fulltextindex", "sync" );
+
+    return settings;
 }
 
 #include "repository.moc"
