@@ -18,6 +18,7 @@
 
 #include "storage.h"
 #include "nepomukcore.h"
+#include "repository.h"
 
 #include <QtDBus/QDBusConnection>
 #include <QtCore/QFile>
@@ -73,17 +74,9 @@ void Nepomuk::Storage::slotNepomukCoreInitialized( bool success )
 }
 
 
-void Nepomuk::Storage::optimize( const QString& repo )
-{
-    m_core->optimize( repo );
-}
-
-
 QString Nepomuk::Storage::usedSopranoBackend() const
 {
-    // FIXME: this is only partly true. It is perfectly possible to change the backend only
-    //        for the main repo in the config file
-    return Repository::activeSopranoBackend()->pluginName();
+    return static_cast<Repository*>( m_core->model( QLatin1String( "main" ) ) )->usedSopranoBackend();
 }
 
 #include "storage.moc"
