@@ -21,6 +21,8 @@
 
 #include "indexscheduler.h"
 #include "strigiserviceconfig.h"
+#include "nfo.h"
+#include "nie.h"
 
 #include <QtCore/QMutexLocker>
 #include <QtCore/QList>
@@ -33,6 +35,14 @@
 
 #include <KDebug>
 #include <KTemporaryFile>
+#include <KUrl>
+
+#include <Nepomuk/ResourceManager>
+
+#include <Soprano/Model>
+#include <Soprano/QueryResultIterator>
+#include <Soprano/NodeIterator>
+#include <Soprano/Node>
 
 #include <map>
 #include <vector>
@@ -46,9 +56,14 @@
 #include <strigi/analyzerconfiguration.h>
 
 
-// FIXME: remove all files from the datastore which are in folders not supposed to be indexed
+namespace {
+    const int s_reducedSpeedDelay = 50; // ms
+    const int s_snailPaceDelay = 200;   // ms
+}
 
-class StoppableConfiguration : public Strigi::AnalyzerConfiguration {
+
+class Nepomuk::IndexScheduler::StoppableConfiguration : public Strigi::AnalyzerConfiguration
+{
 public:
     StoppableConfiguration()
         : m_stop(false) {
@@ -76,33 +91,13 @@ private:
 };
 
 
-namespace {
-    enum UpdateDirFlags {
-        /**
-         * No flags, only used to make code more readable
-         */
-        NoUpdateFlags = 0x0,
-
-        /**
-         * The folder should be updated recursive
-         */
-        UpdateRecursive = 0x1,
-
-        /**
-         * The folder has been scheduled to update by the
-         * update system, not by a call to updateDir
-         */
-        AutoUpdateFolder = 0x2
-    };
-}
-
-
 Nepomuk::IndexScheduler::IndexScheduler( Strigi::IndexManager* manager, QObject* parent )
     : QThread( parent ),
       m_suspended( false ),
       m_stopped( false ),
       m_indexing( false ),
-      m_indexManager( manager )
+      m_indexManager( manager ),
+      m_speed( FullSpeed )
 {
     m_analyzerConfig = new StoppableConfiguration;
 
@@ -119,19 +114,21 @@ Nepomuk::IndexScheduler::~IndexScheduler()
 
 void Nepomuk::IndexScheduler::suspend()
 {
-    if ( isRunning() ) {
+    if ( isRunning() && !m_suspended ) {
         QMutexLocker locker( &m_resumeStopMutex );
         m_suspended = true;
+        emit indexingSuspended( true );
     }
 }
 
 
 void Nepomuk::IndexScheduler::resume()
 {
-    if ( isRunning() ) {
+    if ( isRunning() && m_suspended ) {
         QMutexLocker locker( &m_resumeStopMutex );
         m_suspended = false;
         m_resumeStopWc.wakeAll();
+        emit indexingSuspended( false );
     }
 }
 
@@ -166,6 +163,22 @@ void Nepomuk::IndexScheduler::restart()
 }
 
 
+void Nepomuk::IndexScheduler::setIndexingSpeed( IndexingSpeed speed )
+{
+    kDebug() << speed;
+    m_speed = speed;
+}
+
+
+void Nepomuk::IndexScheduler::setReducedIndexingSpeed( bool reduced )
+{
+    if ( reduced )
+        setIndexingSpeed( ReducedSpeed );
+    else
+        setIndexingSpeed( FullSpeed );
+}
+
+
 bool Nepomuk::IndexScheduler::isSuspended() const
 {
     return isRunning() && m_suspended;
@@ -202,9 +215,6 @@ void Nepomuk::IndexScheduler::run()
     setPriority( QThread::IdlePriority );
 
     // initialization
-    m_suspended = false;
-    m_stopped = false;
-    m_analyzerConfig->setStop( false );
     readConfig();
 
     Strigi::StreamAnalyzer analyzer( *m_analyzerConfig );
@@ -212,10 +222,6 @@ void Nepomuk::IndexScheduler::run()
 
     setIndexingStarted( true );
 
-    // do the actual indexing
-    foreach( const QString& f, StrigiServiceConfig::self()->folders() )
-        m_dirsToUpdate << qMakePair( f, UpdateRecursive|AutoUpdateFolder );
-
     while ( 1 ) {
         // wait for more dirs to analyze in case the initial
         // indexing is done
@@ -237,25 +243,27 @@ void Nepomuk::IndexScheduler::run()
 
         // get the next folder
         m_dirsToUpdateMutex.lock();
-        QPair<QString, int> dir = *m_dirsToUpdate.begin();
+        QPair<QString, UpdateDirFlags> dir = *m_dirsToUpdate.begin();
         m_dirsToUpdate.erase( m_dirsToUpdate.begin() );
         m_dirsToUpdateMutex.unlock();
 
         // update until stopped
-        if ( !updateDir( dir.first, &analyzer, dir.second & UpdateRecursive ) ) {
+        if ( !updateDir( dir.first, &analyzer, dir.second | UpdateRecursive ) ) {
             break;
         }
         m_currentFolder.clear();
     }
 
     setIndexingStarted( false );
+
+    // reset state
+    m_suspended = false;
+    m_stopped = false;
+    m_analyzerConfig->setStop( false );
 }
 
 
-// FIXME: the folders in StrigiServiceConfig::self()->folders() themselves are not indexed.
-
-// this method should be thread-safe ("should" because of the unknown situation with indexreader and -writer)
-bool Nepomuk::IndexScheduler::updateDir( const QString& dir, Strigi::StreamAnalyzer* analyzer, bool recursive )
+bool Nepomuk::IndexScheduler::updateDir( const QString& dir, Strigi::StreamAnalyzer* analyzer, UpdateDirFlags flags )
 {
 //    kDebug() << dir << analyzer << recursive;
 
@@ -263,6 +271,16 @@ bool Nepomuk::IndexScheduler::updateDir( const QString& dir, Strigi::StreamAnaly
     emit indexingFolder( dir );
 
     m_currentFolder = dir;
+    const bool recursive = flags&UpdateRecursive;
+    const bool forceUpdate = flags&ForceUpdate;
+
+    // we start by updating the folder itself
+    time_t storedMTime = m_indexManager->indexReader()->mTime( QFile::encodeName( dir ).data() );
+    QFileInfo dirInfo( dir );
+    if ( storedMTime == 0 ||
+         storedMTime != dirInfo.lastModified().toTime_t() ) {
+        analyzeFile( dirInfo, analyzer );
+    }
 
     // get a map of all indexed files from the dir including their stored mtime
     std::map<std::string, time_t> filesInStore;
@@ -271,7 +289,7 @@ bool Nepomuk::IndexScheduler::updateDir( const QString& dir, Strigi::StreamAnaly
 
     QList<QFileInfo> filesToIndex;
     QList<QString> subFolders;
-    std::vector<std::string> filesToUpdate, filesToDelete;
+    std::vector<std::string> filesToDelete;
 
     // iterate over all files in the dir
     // and select the ones we need to add or delete from the store
@@ -280,7 +298,9 @@ bool Nepomuk::IndexScheduler::updateDir( const QString& dir, Strigi::StreamAnaly
     while ( dirIt.hasNext() ) {
         QString path = dirIt.next();
 
-        QFileInfo fileInfo = dirIt.fileInfo();
+        // FIXME: we cannot use canonialFilePath here since that could lead into another folder. Thus, we probably
+        // need to use another approach then the getChildren one.
+        QFileInfo fileInfo = dirIt.fileInfo();//.canonialFilePath();
 
         bool indexFile = m_analyzerConfig->indexFile( QFile::encodeName( path ), QFile::encodeName( fileInfo.fileName() ) );
 
@@ -291,11 +311,11 @@ bool Nepomuk::IndexScheduler::updateDir( const QString& dir, Strigi::StreamAnaly
         // do we need to update? Did the file change?
         bool fileChanged = !newFile && fileInfo.lastModified().toTime_t() != filesInStoreIt->second;
 
-        if ( indexFile && ( newFile || fileChanged ) )
+        if ( indexFile && ( newFile || fileChanged || forceUpdate ) )
             filesToIndex << fileInfo;
 
-        if ( !newFile && fileChanged )
-            filesToUpdate.push_back( filesInStoreIt->first );
+        // we do not delete files to update here. We do that in the IndexWriter to make
+        // sure we keep the resource URI
         else if ( !newFile && !indexFile )
             filesToDelete.push_back( filesInStoreIt->first );
 
@@ -314,9 +334,6 @@ bool Nepomuk::IndexScheduler::updateDir( const QString& dir, Strigi::StreamAnaly
         filesToDelete.push_back( it->first );
     }
 
-    // remove all files non-recursively that need updating
-    m_indexManager->indexWriter()->deleteEntries( filesToUpdate );
-
     // remove all files that have been removed recursively
     deleteEntries( filesToDelete );
 
@@ -335,7 +352,7 @@ bool Nepomuk::IndexScheduler::updateDir( const QString& dir, Strigi::StreamAnaly
     if ( recursive ) {
         foreach( const QString& folder, subFolders ) {
             if ( !StrigiServiceConfig::self()->excludeFolders().contains( folder ) &&
-                 !updateDir( folder, analyzer, true ) )
+                 !updateDir( folder, analyzer, flags ) )
                 return false;
         }
     }
@@ -346,13 +363,18 @@ bool Nepomuk::IndexScheduler::updateDir( const QString& dir, Strigi::StreamAnaly
 
 void Nepomuk::IndexScheduler::analyzeFile( const QFileInfo& file, Strigi::StreamAnalyzer* analyzer )
 {
-//    kDebug() << file.filePath();
+    //
+    // strigi asserts if the file path has a trailing slash
+    //
+    KUrl url( file.filePath() );
+    QString filePath = url.toLocalFile( KUrl::RemoveTrailingSlash );
+    QString dir = url.directory(KUrl::IgnoreTrailingSlash);
 
-    Strigi::AnalysisResult analysisresult( QFile::encodeName( file.filePath() ).data(),
+    Strigi::AnalysisResult analysisresult( QFile::encodeName( filePath ).data(),
                                            file.lastModified().toTime_t(),
                                            *m_indexManager->indexWriter(),
                                            *analyzer,
-                                           QFile::encodeName( file.path() ).data() );
+                                           QFile::encodeName( dir ).data() );
     if ( file.isFile() && !file.isSymLink() ) {
         Strigi::FileInputStream stream( QFile::encodeName( file.filePath() ) );
         analysisresult.index( &stream );
@@ -371,26 +393,29 @@ bool Nepomuk::IndexScheduler::waitForContinue()
         m_resumeStopWc.wait( &m_resumeStopMutex );
         setIndexingStarted( true );
     }
+    else if ( m_speed != FullSpeed ) {
+        msleep( m_speed == ReducedSpeed ? s_reducedSpeedDelay : s_snailPaceDelay );
+    }
 
     return !m_stopped;
 }
 
 
-void Nepomuk::IndexScheduler::updateDir( const QString& path )
+void Nepomuk::IndexScheduler::updateDir( const QString& path, bool forceUpdate )
 {
     QMutexLocker lock( &m_dirsToUpdateMutex );
-    m_dirsToUpdate << qMakePair( path, ( int )NoUpdateFlags );
+    m_dirsToUpdate << qMakePair( path, UpdateDirFlags( forceUpdate ? ForceUpdate : NoUpdateFlags ) );
     m_dirsToUpdateWc.wakeAll();
 }
 
 
-void Nepomuk::IndexScheduler::updateAll()
+void Nepomuk::IndexScheduler::updateAll( bool forceUpdate )
 {
     QMutexLocker lock( &m_dirsToUpdateMutex );
 
     // remove previously added folders to not index stuff we are not supposed to
     // (FIXME: this does not include currently being indexed folders)
-    QSet<QPair<QString, int> >::iterator it = m_dirsToUpdate.begin();
+    QSet<QPair<QString, UpdateDirFlags> >::iterator it = m_dirsToUpdate.begin();
     while ( it != m_dirsToUpdate.end() ) {
         if ( it->second & AutoUpdateFolder )
             it = m_dirsToUpdate.erase( it );
@@ -398,9 +423,13 @@ void Nepomuk::IndexScheduler::updateAll()
             ++it;
     }
 
+    UpdateDirFlags flags = UpdateRecursive|AutoUpdateFolder;
+    if ( forceUpdate )
+        flags |= ForceUpdate;
+
     // update everything again in case the folders changed
     foreach( const QString& f, StrigiServiceConfig::self()->folders() )
-        m_dirsToUpdate << qMakePair( f, UpdateRecursive|AutoUpdateFolder );
+        m_dirsToUpdate << qMakePair( f, flags );
 
     m_dirsToUpdateWc.wakeAll();
 }
@@ -410,8 +439,8 @@ void Nepomuk::IndexScheduler::readConfig()
 {
     // load Strigi configuration
     std::vector<std::pair<bool, std::string> > filters;
-    QStringList excludeFilters = StrigiServiceConfig::self()->excludeFilters();
-    QStringList includeFilters = StrigiServiceConfig::self()->includeFilters();
+    const QStringList excludeFilters = StrigiServiceConfig::self()->excludeFilters();
+    const QStringList includeFilters = StrigiServiceConfig::self()->includeFilters();
     foreach( const QString& filter, excludeFilters ) {
         filters.push_back( std::make_pair<bool, std::string>( false, filter.toUtf8().data() ) );
     }
@@ -419,6 +448,7 @@ void Nepomuk::IndexScheduler::readConfig()
         filters.push_back( std::make_pair<bool, std::string>( true, filter.toUtf8().data() ) );
     }
     m_analyzerConfig->setFilters(filters);
+    removeOldAndUnwantedEntries();
     updateAll();
 }
 
@@ -502,4 +532,98 @@ void Nepomuk::IndexScheduler::deleteEntries( const std::vector<std::string>& ent
     m_indexManager->indexWriter()->deleteEntries( entries );
 }
 
+
+void Nepomuk::IndexScheduler::removeOldAndUnwantedEntries()
+{
+    //
+    // Get all folders that are stored as parent folders of indexed files
+    //
+    QString query = QString::fromLatin1( "select distinct ?d ?dir where { "
+                                         "?r a %1 . "
+                                         "?g <http://www.strigi.org/fields#indexGraphFor> ?r . "
+                                         "?r %2 ?d . "
+                                         "?d %3 ?dir . }" )
+                    .arg( Soprano::Node::resourceToN3( Nepomuk::Vocabulary::NFO::FileDataObject() ) )
+                    .arg( Soprano::Node::resourceToN3( Nepomuk::Vocabulary::NIE::isPartOf() ) )
+                    .arg( Soprano::Node::resourceToN3( Nepomuk::Vocabulary::NIE::url() ) );
+    Soprano::QueryResultIterator it = ResourceManager::instance()->mainModel()->executeQuery( query, Soprano::Query::QueryLanguageSparql );
+    QList<QPair<KUrl, QUrl> > storedFolders;
+    while ( it.next() && !m_stopped ) {
+        storedFolders << qMakePair( KUrl( it["dir"].uri() ), it["d"].uri() );
+    }
+
+    //
+    // Now compare that list with the configured folders and remove any
+    // entries that are children of folders that should not be indexed.
+    //
+    QList<QUrl> storedFoldersToRemove;
+    for ( int i = 0; i < storedFolders.count(); ++i ) {
+        const KUrl& url = storedFolders[i].first;
+        if ( !StrigiServiceConfig::self()->shouldFolderBeIndexed( url.path() ) ) {
+            storedFoldersToRemove << storedFolders[i].second;
+        }
+    }
+
+    // cleanup
+    storedFolders.clear();
+
+    //
+    // Now we gathered all the folders whose children we need to delete from the storage.
+    // We now need to query all entries in those folders to get a list of graphs we actually
+    // need to delete.
+    //
+    for ( int i = 0; i < storedFoldersToRemove.count() && !m_stopped; ++i ) {
+
+        // wait for resume or stop (or simply continue)
+        if ( !waitForContinue() ) {
+            break;
+        }
+
+        QString query = QString::fromLatin1( "select ?g where { "
+                                             "?r a %1 . "
+                                             "?r %2 %3 . "
+                                             "?g <http://www.strigi.org/fields#indexGraphFor> ?r . }" )
+                        .arg( Soprano::Node::resourceToN3( Nepomuk::Vocabulary::NFO::FileDataObject() ) )
+                        .arg( Soprano::Node::resourceToN3( Nepomuk::Vocabulary::NIE::isPartOf() ) )
+                        .arg( Soprano::Node::resourceToN3( storedFoldersToRemove[i] ) );
+        QList<Soprano::Node> entriesToRemove
+            = ResourceManager::instance()->mainModel()->executeQuery( query, Soprano::Query::QueryLanguageSparql )
+            .iterateBindings( "g" )
+            .allNodes();
+
+        //
+        // Finally delete the entries. The corresponding metadata graphs will be deleted automatically
+        // by the storage service.
+        //
+        for( int j = 0; j < entriesToRemove.count() && !m_stopped; ++j ) {
+
+            // wait for resume or stop (or simply continue)
+            if ( !waitForContinue() ) {
+                break;
+            }
+
+            const Soprano::Node& g = entriesToRemove[j];
+            kDebug() << "Removing old index entry graph" << g;
+            ResourceManager::instance()->mainModel()->removeContext( g );
+        }
+    }
+}
+
+
+QDebug Nepomuk::operator<<( QDebug dbg, IndexScheduler::IndexingSpeed speed )
+{
+    dbg << ( int )speed;
+    switch( speed ) {
+    case IndexScheduler::FullSpeed:
+        return dbg << "FullSpeed";
+    case IndexScheduler::ReducedSpeed:
+        return dbg << "ReducedSpeed";
+    case IndexScheduler::SnailPace:
+        return dbg << "SnailPace";
+    }
+
+    // make gcc shut up
+    return dbg;
+}
+
 #include "indexscheduler.moc"
