@@ -23,6 +23,7 @@
 #include <QtCore/QMutex>
 #include <QtCore/QWaitCondition>
 #include <QtCore/QSet>
+#include <QtCore/QDateTime>
 
 #include <vector>
 #include <string>
@@ -32,10 +33,9 @@ namespace Strigi {
     class IndexManager;
 }
 
-class StoppableConfiguration;
 class QFileInfo;
 class QUrl;
-class QDateTime;
+class KUrl;
 class QByteArray;
 
 
@@ -64,12 +64,74 @@ namespace Nepomuk {
          */
         QString currentFolder() const;
 
+        enum UpdateDirFlag {
+            /**
+             * No flags, only used to make code more readable
+             */
+            NoUpdateFlags = 0x0,
+
+            /**
+             * The folder should be updated recursive
+             */
+            UpdateRecursive = 0x1,
+
+            /**
+             * The folder has been scheduled to update by the
+             * update system, not by a call to updateDir
+             */
+            AutoUpdateFolder = 0x2,
+
+            /**
+             * The files in the folder should be updated regardless
+             * of their state.
+             */
+            ForceUpdate = 0x4
+        };
+        Q_DECLARE_FLAGS( UpdateDirFlags, UpdateDirFlag )
+
+        enum IndexingSpeed {
+            /**
+             * Index at full speed, i.e. do not use any artificial
+             * delays.
+             *
+             * This is the mode used if the user is "away".
+             */
+            FullSpeed = 0,
+
+            /**
+             * Reduce the indexing speed mildly. This is the normal
+             * mode used while the user works. The indexer uses small
+             * delay between indexing two files in order to keep the
+             * load on CPU and IO down.
+             */
+            ReducedSpeed,
+
+            /**
+             * Like ReducedSpeed delays are used but they are much longer
+             * to get even less CPU and IO load. This mode is used for the
+             * first 2 minutes after startup to give the KDE session manager
+             * time to start up the KDE session rapidly.
+             */
+            SnailPace
+        };
+
+        IndexingSpeed currentSpeed() const { return m_speed; }
+
     public Q_SLOTS:
         void suspend();
         void resume();
         void stop();
         void restart();
 
+        void setIndexingSpeed( IndexingSpeed speed );
+
+        /**
+         * A convinience slot which calls setIndexingSpeed
+         * with either FullSpeed or ReducedSpeed, based on the
+         * value of \p reduced.
+         */
+        void setReducedIndexingSpeed( bool reduced = false );
+
         void setSuspended( bool );
 
         /**
@@ -79,12 +141,12 @@ namespace Nepomuk {
          * Updates a complete folder (non-recursively). Makes sense for
          * signals like KDirWatch::dirty.
          */
-        void updateDir( const QString& path );
+        void updateDir( const QString& path, bool forceUpdate = false );
 
         /**
          * Updates all configured folders.
          */
-        void updateAll();
+        void updateAll( bool forceUpdate = false );
 
         /**
          * Analyze a resource that is not read from the local harddisk.
@@ -100,6 +162,7 @@ namespace Nepomuk {
         void indexingStarted();
         void indexingStopped();
         void indexingFolder( const QString& );
+        void indexingSuspended( bool suspended );
 
     private Q_SLOTS:
         void readConfig();
@@ -109,7 +172,7 @@ namespace Nepomuk {
         void run();
 
         bool waitForContinue();
-        bool updateDir( const QString& dir, Strigi::StreamAnalyzer* analyzer, bool recursive );
+        bool updateDir( const QString& dir, Strigi::StreamAnalyzer* analyzer, UpdateDirFlags flags );
         void analyzeFile( const QFileInfo& file, Strigi::StreamAnalyzer* analyzer );
 
         /**
@@ -122,6 +185,12 @@ namespace Nepomuk {
         // no signal is emitted twice
         void setIndexingStarted( bool started );
 
+        /**
+         * Removes all previously indexed entries that are not in the list of folders
+         * to index anymore.
+         */
+        void removeOldAndUnwantedEntries();
+
         bool m_suspended;
         bool m_stopped;
         bool m_indexing;
@@ -129,17 +198,24 @@ namespace Nepomuk {
         QMutex m_resumeStopMutex;
         QWaitCondition m_resumeStopWc;
 
+        class StoppableConfiguration;
         StoppableConfiguration* m_analyzerConfig;
         Strigi::IndexManager* m_indexManager;
 
         // set of folders to update (+flags defined in the source file) - changed by updateDir
-        QSet<QPair<QString, int> > m_dirsToUpdate;
+        QSet<QPair<QString, UpdateDirFlags> > m_dirsToUpdate;
 
         QMutex m_dirsToUpdateMutex;
         QWaitCondition m_dirsToUpdateWc;
 
         QString m_currentFolder;
+
+        IndexingSpeed m_speed;
     };
+
+    QDebug operator<<( QDebug dbg, IndexScheduler::IndexingSpeed speed );
 }
 
+Q_DECLARE_OPERATORS_FOR_FLAGS(Nepomuk::IndexScheduler::UpdateDirFlags)
+
 #endif
